package cc.fascinated.model.user;

import cc.fascinated.common.DateUtils;
import cc.fascinated.model.token.scoresaber.ScoreSaberAccountToken;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Date;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor
@Getter
public class ScoreSaberAccount {
    /**
     * The avatar of the user.
     */
    private String avatar;

    /**
     * The country of the user.
     */
    private String country;

    /**
     * The rank of the user.
     */
    private int rank;

    /**
     * The country rank of the user.
     */
    private int countryRank;

    /**
     * The date the user joined ScoreSaber.
     */
    private Date accountCreated;

    /**
     * The date the user was last updated.
     */
    private Date lastUpdated;

    /**
     * Constructs a new {@link ScoreSaberAccount} object
     * from a {@link ScoreSaberAccountToken} object.
     *
     * @param token The token to construct the object from.
     * @return The scoresaber account.
     */
    public static ScoreSaberAccount getFromToken(ScoreSaberAccountToken token) {
        return new ScoreSaberAccount(
                token.getProfilePicture(),
                token.getCountry(),
                token.getRank(),
                token.getCountryRank(),
                DateUtils.getDateFromIsoString(token.getFirstSeen()),
                new Date()
        );
    }
}
