package cc.fascinated.model.user.history;

import cc.fascinated.common.DateUtils;
import com.fasterxml.jackson.annotation.JsonIgnore;

import java.util.*;

/**
 * @author Fascinated (fascinated7)
 */
public class History {
    /**
     * The user's history points in time.
     */
    private Map<String, HistoryPoint> histories;

    /**
     * The user's history points history.
     */
    @JsonIgnore
    public Map<Date, HistoryPoint> getHistories() {
        if (this.histories == null) {
            this.histories = new HashMap<>();
        }
        Map<Date, HistoryPoint> toReturn = new HashMap<>();
        this.histories.forEach((key, value) -> toReturn.put(DateUtils.getDateFromString(key), value));
        return toReturn;
    }

    /**
     * Gets the user's history for today.
     *
     * @return the user's history for today
     */
    @JsonIgnore
    public HistoryPoint getTodayHistory() {
        if (this.histories == null) {
            this.histories = new HashMap<>();
        }
        Date midnight = DateUtils.getMidnightToday();
        return this.histories.computeIfAbsent(DateUtils.formatDate(midnight), key -> new HistoryPoint());
    }

    /**
     * Gets the user's history for a specific date.
     *
     * @param date the date to get the history for
     * @return the user's history for the date
     */
    public HistoryPoint getHistoryForDate(Date date) {
        if (this.histories == null) {
            this.histories = new HashMap<>();
        }
        return this.histories.get(DateUtils.formatDate(date));
    }

    /**
     * Gets the user's HistoryPoint history for
     * an amount of days ago.
     *
     * @param days the amount of days ago
     * @return the user's HistoryPoint history
     */
    public TreeMap<String, HistoryPoint> getPreviousHistories(int days) {
        Date date = DateUtils.getDaysAgo(days);
        Map<String, HistoryPoint> toReturn = new HashMap<>();
        for (Map.Entry<Date, HistoryPoint> history : getHistories().entrySet()) {
            if (history.getKey().after(date)) {
                toReturn.put(DateUtils.formatDate(history.getKey()), history.getValue());
            }
        }

        // Sort the history by date (newest > oldest)
        TreeMap<String, HistoryPoint> sorted = new TreeMap<>(Comparator.comparing(DateUtils::getDateFromString).reversed());
        sorted.putAll(toReturn);
        return sorted;
    }
}
