package cc.fascinated.websocket;

import lombok.Getter;
import lombok.NonNull;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.client.standard.StandardWebSocketClient;
import org.springframework.web.socket.handler.TextWebSocketHandler;

/**
 * @author Fascinated (fascinated7)
 */
@Log4j2
@Getter
public abstract class Websocket extends TextWebSocketHandler {
    /**
     * The name of the WebSocket.
     */
    private final String name;

    /**
     * The URL of the WebSocket.
     */
    private final String url;

    /**
     * The WebSocket session.
     */
    private WebSocketSession webSocketSession;

    public Websocket(@NonNull String name, @NonNull String url) {
        this.name = name;
        this.url = url;
        connectWebSocket(); // Connect to the WebSocket.
    }

    /**
     * Handles a message received from the WebSocket.
     *
     * @param message the message received
     */
    public abstract void handleMessage(@NonNull TextMessage message);

    @Override
    protected final void handleTextMessage(@NonNull WebSocketSession session, @NonNull TextMessage message) {
        this.handleMessage(message);
    }

    /**
     * Connects to the ScoreSaber WebSocket.
     */
    @SneakyThrows
    private void connectWebSocket() {
        log.info("Connecting to the {}", this.getName());
        this.webSocketSession = new StandardWebSocketClient().execute(this, this.getUrl()).get();
    }

    @Override
    public final void afterConnectionEstablished(@NonNull WebSocketSession session) {
        log.info("Connected to the {}", this.getName());
    }

    @Override
    public final void afterConnectionClosed(@NonNull WebSocketSession session, @NonNull CloseStatus status) {
        log.info("Disconnected from the {}", this.getName());
        connectWebSocket(); // Reconnect to the WebSocket.
    }
}
